var _ = require('lodash');
var fs = require('fs');
var path = require('path');
var fse = require('fs-extra');
var closest = require('closest-package');
var crypto = require('crypto');
var DocRegistry = (function () {
    function DocRegistry() {
        this.docs = {};
    }
    DocRegistry.prototype.addDoc = function (fileName, doc) {
        this.docs[fileName] = doc;
    };
    DocRegistry.prototype.writeDocs = function (dir) {
        fse.ensureDirSync(dir);
        _.forEach(this.docs, function (doc) {
            var metaPath = path.join(dir, doc.fileInfo.metaName);
            fs.writeFileSync(metaPath, JSON.stringify(doc, null, 4));
        });
        this.writeRegistryModule(dir);
    };
    DocRegistry.prototype.generateRegistryModule = function (dir) {
        var buf = "\nmodule.exports = {\n\n    mainPackage: '" + extractPackage(dir).info.name + "',\n    files: {\n        ";
        _.forEach(this.docs, function (doc) {
            buf += "    '" + doc.fileInfo.metaName + "': require('./" + doc.fileInfo.metaName + "'),\n";
        });
        buf += '}}';
        return buf;
    };
    DocRegistry.prototype.writeRegistryModule = function (dir) {
        var registryModule = this.generateRegistryModule(dir);
        fs.writeFileSync('./docs/registry.js', registryModule);
    };
    return DocRegistry;
})();
exports.DocRegistry = DocRegistry;
function extractPackage(fileName) {
    var pkgJson = closest.sync(path.dirname(fileName));
    return {
        path: path.dirname(pkgJson),
        info: JSON.parse(fs.readFileSync(pkgJson).toString())
    };
}
exports.extractPackage = extractPackage;
function getDocFilePath(fileName, pkg) {
    var relativeToOrigin = path.relative(process.cwd(), fileName);
    var shasum = crypto.createHash('sha1');
    shasum.update(relativeToOrigin);
    var metaName = shasum.digest('hex') + '.json';
    var relativeToPackage = path.relative(pkg.path, fileName);
    if (!/^(\.|\/)/.test(relativeToPackage)) {
        relativeToPackage = '/' + relativeToPackage;
    }
    return {
        metaName: metaName,
        relativeToOrigin: relativeToOrigin,
        relativeToPackage: relativeToPackage
    };
}
exports.getDocFilePath = getDocFilePath;
var Doc = (function () {
    function Doc(sourceFile, pkg, fileInfo) {
        this.text = sourceFile.text;
        this.pkg = {
            name: pkg.info.name,
            version: pkg.info.version,
            description: pkg.info.description,
        };
        this.fileInfo = fileInfo;
    }
    Doc.prototype.toJSON = function () {
        var _a = this, text = _a.text, pkg = _a.pkg, fileInfo = _a.fileInfo;
        return { pkg: pkg, fileInfo: fileInfo, text: text };
    };
    return Doc;
})();
function generateDoc(fileName, source) {
    var pkg = extractPackage(fileName);
    var docFilePath = getDocFilePath(fileName, pkg);
    var doc = new Doc(source, pkg, docFilePath);
    return doc;
}
exports.generateDoc = generateDoc;
//# sourceMappingURL=doc.js.map